#!/bin/sh
#*******************************************************************************************************************************
# This script is provided to use with Areca ARC-11XX/12XX/16XX/18XX SAS/SATA RAID controllers' linux device driver.
#
# This script runs on a specified host and target and modifies 
# the scsi device timeout for all luns (scsi devices). 
# This can help when target devices take longer time to execute a command,
# such as under heavy IO or hard disks' instability.
# Setting a longer timeout value reduces the chance of the Linux SCSI Midlayer
# driver aborting the tasks after this timeout threshold.
# This script should be run manually every time after the system reboots.
# If you don't, the scsi device timeout value will return to the default.
#
# USAGE: 
# The script is invoked with at least 3 arguments. The arguments
# specify the SCSI Host number, the target id managed by that host, and
# the new timeout value desired.  Since modifying scsi device timeouts
# is very target/load sensitive, the sys admin must execute this 
# script per host per target.
# 
# Note:
# /sys/class/scsi_device/H:B:T:L where 'H' is the host no, 'B' is the
# bus/channel number, 'T' is the target/id number and 'L' is the LUN
#
# On the other hand, for saving time, you can add the entry,
# echo "timeout_value" > /sys/class/scsi_device/H:B:T:L/device/timeout in ~/.bashrc,
# where 'H' is the host no, 'B' is the bus/channel number, 'T' is the target/id number, 'L' is the LUN
# and timeout_value is the desired scsi device timeout value in second. Then, the timeout value will be modified as you set
# automatically as you reboot.
# Before you proceed with this step, be sure which mapping scsi ID your target device is.
#*******************************************************************************************************************************
usage()
{
        echo ""
        echo "  Usage: set_scsicmd_timeout [ host_num target_id cmd_timeout ]"
        echo "  where "
        echo "    host_num    : is the scsi host number assigned to the Areca driver instance "
        echo "                  providing access to the luns."
        echo "    target_id   : is the target id of the target containing the scsi devices(luns)"
        echo "    cmd_timeout : is the new timeout value desired"
        echo ""
        echo "  Example:"
        echo "    set_scsicmd_timeout.sh 2 1 90  : Set the scsi command timeout on host 2, "
		echo "    target id 1 to 90 seconds for all luns contained on this target"
        echo ""
        echo ""
}

# Validate argument list
if [ $# -eq 0 -o "$1" = "--help" -o "$1" = "-h" ] ; then
        usage
        exit 1
fi

# Enter sysfs for the scsi device class first.
cd /sys/class/scsi_device/

# Change the per command timeout value on all luns
# for the given host, channel(always =0) , and target.

for i in `ls -d $1:0:$2*`
do
cd $i/device
echo "modifying device `pwd`"
echo "found timeout at value `cat timeout`"
echo $3 > timeout
echo "new timeout value is `cat timeout`"
cd ../..
done
